@@ -206,3 +206,30 @@ class MidonetApiClient(base.MidonetClientBase):
 
     def delete_firewall_log_postcommit(self, firewall_log_id):
         self.api_cli.delete_firewall_log(firewall_log_id)
+
+    def create_tap_flow(self, context, tap_flow):
+        self.api_cli.create_tap_flow(tap_flow)
+
+    def delete_tap_flow(self, context, tap_flow_id):
+        self.api_cli.delete_tap_flow(tap_flow_id)
+
+    def create_tap_service(self, context, tap_service):
+        self.api_cli.create_tap_service(tap_service)
+
+    def delete_tap_service(self, context, tap_service_id):
+        self.api_cli.delete_tap_service(tap_service_id)
+
+    # QoS
+    def create_qos_policy(self, context, policy):
+        policy_dict = policy.to_dict()
+        self.api_cli.create_qos_policy(policy_dict)
+
+    def update_qos_policy(self, context, policy):
+        policy_dict = policy.to_dict()
+        id_ = policy_dict['id']
+        self.api_cli.update_qos_policy(id_, policy_dict)
+
+    def delete_qos_policy(self, context, policy):
+        policy_dict = policy.to_dict()
+        id_ = policy_dict['id']
+        self.api_cli.delete_qos_policy(id_)
