@@ -13,14 +13,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib import constants as n_const
+from neutron_lib import exceptions as n_exc
+
 from midonet.neutron._i18n import _, _LE, _LW
 from midonet.neutron.common import utils as c_utils
 from midonet.neutron.db import agent_membership_db as am_db
 from midonet.neutron.db import port_binding_db as pb_db
 from midonet.neutron.db import provider_network_db as pnet_db
 from midonet.neutron import plugin
-from neutron.common import constants as n_const
-from neutron.common import exceptions as n_exc
 from neutron.db import allowedaddresspairs_db as addr_pair_db
 from neutron.db import api as db_api
 from neutron.db import portsecurity_db as ps_db
@@ -68,6 +69,8 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
     ]
 
     __native_bulk_support = True
+    __native_pagination_support = True
+    __native_sorting_support = True
 
     def __init__(self):
         super(MidonetPluginV2, self).__init__()
@@ -82,7 +85,9 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
         self._ensure_default_security_group(context, tenant_id)
 
         with context.session.begin(subtransactions=True):
-            net = super(MidonetPluginV2, self).create_network(context, network)
+            net_db = self.create_network_db(context, network)
+            net = self._make_network_dict(net_db, process_extensions=False,
+                                          context=context)
             net_data['id'] = net['id']
             self._process_l3_create(context, net, net_data)
             self._create_provider_network(context, net_data)
@@ -105,10 +110,7 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
                     LOG.exception(_LE("Failed to delete network %s"),
                                   net['id'])
 
-        # NOTE(kevinbenton): this extra lookup is necessary to get the
-        # latest db model for the extension functions
-        net_model = self._get_network(context, net['id'])
-        self._apply_dict_extend_functions('networks', net, net_model)
+        self._apply_dict_extend_functions('networks', net, net_db)
 
         LOG.debug("MidonetPluginV2.create_network exiting: net=%r", net)
         return net
@@ -157,6 +159,10 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
                 self._process_network_port_security_update(context,
                                                            network['network'],
                                                            net)
+            # NOTE(yamamoto): Retrieve the db object to get the correct
+            # revision
+            context.session.flush()
+            net = self.get_network(context, id)
             self.client.update_network_precommit(context, id, net)
 
         self.client.update_network_postcommit(id, net)
@@ -223,6 +229,10 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
 
         with context.session.begin(subtransactions=True):
             s = super(MidonetPluginV2, self).update_subnet(context, id, subnet)
+            # NOTE(yamamoto): Retrieve the db object to get the correct
+            # revision
+            context.session.flush()
+            s = self.get_subnet(context, id)
             self.client.update_subnet_precommit(context, id, s)
 
         self.client.update_subnet_postcommit(id, s)
@@ -236,14 +246,14 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
         port_data = port['port']
         # REVISIT(yamamoto): this nested transaction is a workaround
         # for bug #1490917.
-        with db_api.exc_to_retry(oslo_db_exc.DBDuplicateEntry),\
-                db_api.autonested_transaction(context.session):
+        with db_api.autonested_transaction(context.session):
             # Set status along admin_state_up if the parameter is specified.
             if port['port'].get('admin_state_up') is not None:
                 if not port['port']['admin_state_up']:
                     port['port']['status'] = n_const.PORT_STATUS_DOWN
             # Create a Neutron port
-            new_port = super(MidonetPluginV2, self).create_port(context, port)
+            port_db = self.create_port_db(context, port)
+            new_port = self._make_port_dict(port_db, process_extensions=False)
 
             # Do not create a gateway port if it has no IP address assigned as
             # MidoNet does not yet handle this case.
@@ -312,10 +322,7 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
                     LOG.exception(_LE("Failed to delete port %s"),
                                   new_port['id'])
 
-        # NOTE(kevinbenton): this extra lookup is necessary to get the
-        # latest db model for the extension functions
-        port_model = self._get_port(context, new_port['id'])
-        self._apply_dict_extend_functions('ports', new_port, port_model)
+        self._apply_dict_extend_functions('ports', new_port, port_db)
 
         LOG.debug("MidonetPluginV2.create_port exiting: port=%r", new_port)
         return new_port
@@ -348,8 +355,7 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
         LOG.debug("MidonetPluginV2.update_port called: id=%(id)s "
                   "port=%(port)r", {'id': id, 'port': port})
 
-        with db_api.exc_to_retry(oslo_db_exc.DBDuplicateEntry),\
-                context.session.begin(subtransactions=True):
+        with context.session.begin(subtransactions=True):
 
             # update the port DB
             original_port = super(MidonetPluginV2, self).get_port(context, id)
@@ -381,7 +387,11 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
                     raise psec.PortSecurityPortHasSecurityGroup()
                 if p.get(addr_pair.ADDRESS_PAIRS):
                     raise addr_pair.AddressPairAndPortSecurityRequired()
-
+            # NOTE(yamamoto): Retrieve the db object to get the correct
+            # revision
+            context.session.flush()
+            context.session.expire_all()
+            p = self.get_port(context, id)
             self.client.update_port_precommit(context, id, p)
 
         # Set status along admin_state_up to update.
@@ -393,9 +403,12 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
         try:
             self.client.update_port_postcommit(id, p)
             data = {'port': {'status': update_status}}
-            super(MidonetPluginV2, self).update_port(context, id, data)
+            new_port = super(MidonetPluginV2, self).update_port(
+                context, id, data)
             # prevent binding:profile information from lacking
-            p['status'] = data['port']['status']
+            p['status'] = new_port['status']
+            if 'revision_number' in new_port:
+                p['revision_number'] = new_port['revision_number']
         except Exception as ex:
             with excutils.save_and_reraise_exception():
                 LOG.error(_LE("Failed to update a port %(port_id)s in "
@@ -603,51 +616,3 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
         if not agent:
             agent = super(MidonetPluginV2, self).get_agent(context, id, fields)
         return agent
-
-    # Patch PortSecurityDbCommon to fix up the lack of DB rows at runtime.
-    # See bug/1507651 for details.
-    # REVISIT(yamamoto): The most of these should be fixed in the base class
-
-    def _extend_port_security_dict(self, response_data, db_data):
-        # NOTE(yamamoto): This method is used for both of networks and ports
-        if db_data.get('port_security') is None:
-            response_data[psec.PORTSECURITY] = (
-                psec.EXTENDED_ATTRIBUTES_2_0['networks']
-                [psec.PORTSECURITY]['default'])
-        else:
-            response_data[psec.PORTSECURITY] = (
-                                db_data['port_security'][psec.PORTSECURITY])
-
-    def _get_network_security_binding(self, context, network_id):
-        # NOTE(yamamoto): used via _determine_port_security_and_has_ip
-        try:
-            return super(MidonetPluginV2,
-                         self)._get_network_security_binding(context,
-                                                             network_id)
-        except psec.PortSecurityBindingNotFound:
-            return (psec.EXTENDED_ATTRIBUTES_2_0['networks']
-                    [psec.PORTSECURITY]['default'])
-
-    def _get_port_security_binding(self, context, network_id):
-        # NOTE(yamamoto): Currently not used by our plugin.
-        # Override just to detect future problems earlier.
-        raise NotImplementedError()
-
-    def _process_network_port_security_update(self, context, network_req,
-                                              network_res):
-        try:
-            super(MidonetPluginV2, self)._process_network_port_security_update(
-                context, network_req, network_res)
-        except psec.PortSecurityBindingNotFound:
-            self._process_network_port_security_create(context, network_req,
-                                                       network_res)
-
-    def _process_port_port_security_update(self, context, port_req, port_res):
-        try:
-            super(MidonetPluginV2, self)._process_port_port_security_update(
-                context, port_req, port_res)
-        except psec.PortSecurityBindingNotFound:
-            self._process_port_port_security_create(context, port_req,
-                                                    port_res)
-
-    # End of patching
