@@ -13,19 +13,23 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib import constants as n_const
+from neutron_lib import exceptions as n_exc
+
 from midonet.neutron._i18n import _LE, _LW
 from midonet.neutron.client import base as c_base
 from midonet.neutron.common import config  # noqa
 from midonet.neutron.common import constants as m_const
 from midonet.neutron.db import l3_db_midonet
 from midonet.neutron import extensions
+from midonet.neutron.extensions import routerinterfacefip
 
 from neutron.api import extensions as neutron_extensions
-from neutron.common import constants as n_const
-from neutron.common import exceptions as n_exc
+from neutron.callbacks import events
+from neutron.callbacks import registry
+from neutron.callbacks import resources
 from neutron.db import common_db_mixin
 from neutron.db import extraroute_db
-from neutron.db import l3_db
 # Import l3_dvr_db to get the config options required for FWaaS
 from neutron.db import l3_dvr_db  # noqa
 from neutron.extensions import l3
@@ -48,11 +52,15 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
     Implements L3 Router service plugin for Midonet.
     """
 
-    supported_extension_aliases = ["router", "extraroute", "ext-gw-mode"]
+    supported_extension_aliases = ["router", "extraroute", "ext-gw-mode",
+                                   "router-interface-fip"]
+
+    __native_pagination_support = True
+    __native_sorting_support = True
 
     def __init__(self):
         super(MidonetL3ServicePlugin, self).__init__()
-        l3_db.subscribe()
+        self.__subscribe()
 
         # Instantiate MidoNet API client
         self.client = c_base.load_client(cfg.CONF.MIDONET)
@@ -61,7 +69,8 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
         cfg.CONF.set_override("router_distributed", False)
         neutron_extensions.append_api_extensions_path(extensions.__path__)
 
-    def get_plugin_type(self):
+    @classmethod
+    def get_plugin_type(cls):
         return constants.L3_ROUTER_NAT
 
     def get_plugin_description(self):
@@ -96,6 +105,9 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
     @log_helpers.log_method_call
     def create_router(self, context, router):
         with context.session.begin(subtransactions=True):
+            # REVISIT(yamamoto): This should not call create_port inside
+            # of a transaction.
+            setattr(context, 'GUARD_TRANSACTION', False)
             r = super(MidonetL3ServicePlugin, self).create_router(context,
                                                                   router)
             self._validate_router_gw_network(context, r)
@@ -116,6 +128,10 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
     @log_helpers.log_method_call
     def update_router(self, context, id, router):
         with context.session.begin(subtransactions=True):
+            # REVISIT(yamamoto): Updating external_gateway_info causes
+            # create_port/delete_port.  This should not call them inside of
+            # a transaction.
+            setattr(context, 'GUARD_TRANSACTION', False)
             r = super(MidonetL3ServicePlugin, self).update_router(context, id,
                                                                   router)
             self._validate_router_gw_network(context, r)
@@ -145,6 +161,9 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
         self._check_router_not_in_use(context, id)
 
         with context.session.begin(subtransactions=True):
+            # REVISIT(yamamoto): This should not call delete_port inside
+            # of a transaction.
+            setattr(context, 'GUARD_TRANSACTION', False)
             super(MidonetL3ServicePlugin, self).delete_router(context, id)
             self.client.delete_router_precommit(context, id)
 
@@ -154,6 +173,9 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
     def add_router_interface(self, context, router_id, interface_info):
         by_port = bool(interface_info.get('port_id'))
         with context.session.begin(subtransactions=True):
+            # REVISIT(yamamoto): This should not call create_port/update_port
+            # inside of a transaction.
+            setattr(context, 'GUARD_TRANSACTION', False)
             info = super(MidonetL3ServicePlugin, self).add_router_interface(
                 context, router_id, interface_info)
             self._validate_network_type(context, info['network_id'])
@@ -176,6 +198,9 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
     @log_helpers.log_method_call
     def remove_router_interface(self, context, router_id, interface_info):
         with context.session.begin(subtransactions=True):
+            # REVISIT(yamamoto): This should not call delete_port inside
+            # of a transaction.
+            setattr(context, 'GUARD_TRANSACTION', False)
             info = super(MidonetL3ServicePlugin, self).remove_router_interface(
                 context, router_id, interface_info)
             self.client.remove_router_interface_precommit(context, router_id,
@@ -207,6 +232,9 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
     @log_helpers.log_method_call
     def delete_floatingip(self, context, id):
         with context.session.begin(subtransactions=True):
+            # REVISIT(yamamoto): This should not call delete_port inside
+            # of a transaction.
+            setattr(context, 'GUARD_TRANSACTION', False)
             super(MidonetL3ServicePlugin, self).delete_floatingip(context, id)
             self.client.delete_floatingip_precommit(context, id)
 
@@ -217,21 +245,6 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
         with context.session.begin(subtransactions=True):
             fip = super(MidonetL3ServicePlugin, self).update_floatingip(
                 context, id, floatingip)
-            port_id = fip['port_id']
-            if port_id is not None:
-                port = self._core_plugin.get_port(context, port_id)
-                owner = port['device_owner']
-                # REVISIT(yamamoto): Empty owner is allowed for tempest and
-                # unit tests.
-                # NOTE(yamamoto): VIP is allowed for non MidoNet LB providers
-                if (owner and
-                   not owner.startswith(n_const.DEVICE_OWNER_COMPUTE_PREFIX)
-                   and owner != n_const.DEVICE_OWNER_LOADBALANCER):
-                    raise n_exc.UnsupportedPortDeviceOwner(
-                        op='floatingip association',
-                        port_id=port_id,
-                        device_owner=owner)
-
             self.client.update_floatingip_precommit(context, id, fip)
 
             # Update status based on association
@@ -255,3 +268,18 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
                     LOG.exception(_LE("Failed to update floating ip "
                                       "status %s"), id)
         return fip
+
+    def _check_router_interface_used_as_gw_for_fip(self, resource,
+                                                   event, trigger, **kwargs):
+        context = kwargs['context']
+        router_id = kwargs['router_id']
+        subnet_id = kwargs['subnet_id']
+        if self._subnet_has_fip(context, router_id, subnet_id):
+            raise routerinterfacefip.RouterInterfaceInUseAsGatewayByFloatingIP(
+                router_id=router_id, subnet_id=subnet_id)
+
+    def __subscribe(self):
+        registry.subscribe(
+            self._check_router_interface_used_as_gw_for_fip,
+            resources.ROUTER_INTERFACE,
+            events.BEFORE_DELETE)
